--- old/usr/src/uts/common/fs/zfs/dmu_objset.c	2015-06-19 17:15:05.365046489 +0200
+++ new/usr/src/uts/common/fs/zfs/dmu_objset.c	2015-06-19 17:15:05.369036478 +0200
@@ -1746,7 +1746,19 @@
 	dmu_objset_find_ctx_t *dcp = arg;
 	dsl_pool_t *dp = dcp->dc_dp;
 
-	dsl_pool_config_enter(dp, FTAG);
+	/*
+	 * We need to get a pool_config_lock here, as there are several
+	 * asssert(pool_config_held) down the stack. Getting a lock via
+	 * dsl_pool_config_enter is risky, as it might be stalled by a
+	 * pending writer. This would deadlock, as the write lock can
+	 * only be granted when our parent thread gives up the lock.
+	 * The _prio interface gives us priority over a pending writer.
+	 * On the other hand, we don't risk to stall any pending writers,
+	 * as the parent thread already holds a config lock. We give up
+	 * our lock before the parent does, so in effect we do not prolong
+	 * the waiting time for the writer.
+	 */
+	dsl_pool_config_enter_prio(dp, FTAG);
 
 	dmu_objset_find_dp_impl(dcp);
 
